﻿/*******************************************************************************
* Copyright (C) 2016 Maxim Integrated Products, Inc., All rights Reserved.
* 
* This software is protected by copyright laws of the United States and
* of foreign countries. This material may also be protected by patent laws
* and technology transfer regulations of the United States and of foreign
* countries. This software is furnished under a license agreement and/or a
* nondisclosure agreement and may only be used or reproduced in accordance
* with the terms of those agreements. Dissemination of this information to
* any party or parties not specified in the license agreement and/or
* nondisclosure agreement is expressly prohibited.
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL MAXIM INTEGRATED BE LIABLE FOR ANY CLAIM, DAMAGES
* OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.
*
* Except as contained in this notice, the name of Maxim Integrated
* Products, Inc. shall not be used except as stated in the Maxim Integrated
* Products, Inc. Branding Policy.
*
* The mere transfer of this software does not imply any licenses
* of trade secrets, proprietary technology, copyrights, patents,
* trademarks, maskwork rights, or any other form of intellectual
* property whatsoever. Maxim Integrated Products, Inc. retains all
* ownership rights.
*******************************************************************************
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Windows.Forms;

using Maxim.Global;

namespace Maxim.Utility
{
    public class FileLog
    {
        StreamWriter outputStream = null;
        SaveFileDialog fileDialog = new SaveFileDialog();
        string fileDirectory = null;

        public EventHandler<Maxim.Global.ErrorEventArgs> Error;

        public FileLog()
        {
            fileDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            fileDialog.InitialDirectory = fileDirectory;
        }

        public string FileDirectory
        {
            get
            {
                return fileDirectory;
            }
            set
            {
                fileDirectory = value;
            }
        }
        
        public bool SelectCSVFile(string fileName)
        {
            fileDialog.FileName = fileName + "_" + string.Format("{0:yyyy-MM-dd_HH-mm-ss}", DateTime.Now) + ".csv";
            fileDialog.Filter = "CSV files (*.csv)|*.csv|All files (*.*)|*.*";
            fileDialog.InitialDirectory = fileDirectory;
            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                fileDirectory = Path.GetDirectoryName(fileDialog.FileName);
                outputStream = new StreamWriter(fileDialog.FileName, false, System.Text.Encoding.UTF8);
                outputStream.AutoFlush = true;

                return true;
            }
            else
                return false;
        }

        public void Close()
        {
            if (outputStream != null)
            {
                try
                {
                    outputStream.Flush();
                }
                catch (System.ObjectDisposedException)
                {
                    // TODO
                }
                outputStream.Close();
            }
        }

        public void Write(string value)
        {
            outputStream.Write(value);
        }

        public void WriteLine(string value)
        {
            //outputStream.WriteLine(string.Format("{0:HH-mm-ss}", DateTime.Now) + ", " + value);
            outputStream.WriteLine(value);
        }
    }
}
