﻿/*******************************************************************************
* Copyright (C) 2016 Maxim Integrated Products, Inc., All rights Reserved.
* 
* This software is protected by copyright laws of the United States and
* of foreign countries. This material may also be protected by patent laws
* and technology transfer regulations of the United States and of foreign
* countries. This software is furnished under a license agreement and/or a
* nondisclosure agreement and may only be used or reproduced in accordance
* with the terms of those agreements. Dissemination of this information to
* any party or parties not specified in the license agreement and/or
* nondisclosure agreement is expressly prohibited.
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL MAXIM INTEGRATED BE LIABLE FOR ANY CLAIM, DAMAGES
* OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.
*
* Except as contained in this notice, the name of Maxim Integrated
* Products, Inc. shall not be used except as stated in the Maxim Integrated
* Products, Inc. Branding Policy.
*
* The mere transfer of this software does not imply any licenses
* of trade secrets, proprietary technology, copyrights, patents,
* trademarks, maskwork rights, or any other form of intellectual
* property whatsoever. Maxim Integrated Products, Inc. retains all
* ownership rights.
*******************************************************************************
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HealthSensorPlatform.DeviceDescriptions;
using RPCSupport.DeviceSupport;

namespace Maxim.CustomControls
{
    public class DeviceController
    {
        //Hid_BaseClass myHID;
        RegisterInfo[] infoArray;
        Byte deviceAddress;
        Device device;
        RPCSupport.RPCClient rpcClient;

        public Device Device
        {
            get
            {
                return device;
            }
        }

        public event EventHandler<RegisterEventArgs> RegisterRead;
        public event EventHandler<RegisterEventArgs> RegisterWrite;

        public class RegisterEventArgs : EventArgs
        {
            public RegisterInfo reg { get; set; }
        }

        protected virtual void OnRegisterRead(RegisterEventArgs e)
        {
            EventHandler<RegisterEventArgs> handler = RegisterRead;
            if (handler != null)
            {
                handler(this, e);
            }
        }
        protected virtual void OnRegisterWrite(RegisterEventArgs e)
        {
            EventHandler<RegisterEventArgs> handler = RegisterWrite;
            if (handler != null)
            {
                handler(this, e);
            }
        }

        public void SetRegisterInfo(Device device)
        {
            this.device = device;
            this.infoArray = device.Info;
        }
        //public void SetHid(Hid_BaseClass hid) {
        //    myHID = hid;
        //}
        public void SetDeviceAddress(Byte address)
        {
            deviceAddress = address;
        }
        public RegisterInfo[] InfoArray
        {
            get
            {
                return infoArray;
            }
        }

        public void ReadAll()
        {
            foreach (RegisterInfo reg in infoArray)
            {
                ReadRegister(reg);
            }
        }

        public void WriteAll()
        {
            foreach (RegisterInfo reg in infoArray)
            {
                WriteRegister(reg);
            }
        }

        public void ReadRegister(RegisterInfo reg)
        {
            rpcClient.ReadRegister(device.deviceDetails, reg);
            //RegisterEventArgs registerEventArgs = new RegisterEventArgs() { reg = reg };
            //OnRegisterRead(registerEventArgs);
        }

        public void WriteRegister(RegisterInfo reg)
        {
            rpcClient.WriteRegister(device.deviceDetails, reg);
            //RegisterEventArgs registerEventArgs = new RegisterEventArgs() { reg = reg };
            //OnRegisterWrite(registerEventArgs);
        }

        internal void SetInterface(RPCSupport.RPCClient rpcClient)
        {
            this.rpcClient = rpcClient;
        }
    }
}
