﻿/*******************************************************************************
* Copyright (C) 2016 Maxim Integrated Products, Inc., All rights Reserved.
* 
* This software is protected by copyright laws of the United States and
* of foreign countries. This material may also be protected by patent laws
* and technology transfer regulations of the United States and of foreign
* countries. This software is furnished under a license agreement and/or a
* nondisclosure agreement and may only be used or reproduced in accordance
* with the terms of those agreements. Dissemination of this information to
* any party or parties not specified in the license agreement and/or
* nondisclosure agreement is expressly prohibited.
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL MAXIM INTEGRATED BE LIABLE FOR ANY CLAIM, DAMAGES
* OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
* OTHER DEALINGS IN THE SOFTWARE.
*
* Except as contained in this notice, the name of Maxim Integrated
* Products, Inc. shall not be used except as stated in the Maxim Integrated
* Products, Inc. Branding Policy.
*
* The mere transfer of this software does not imply any licenses
* of trade secrets, proprietary technology, copyrights, patents,
* trademarks, maskwork rights, or any other form of intellectual
* property whatsoever. Maxim Integrated Products, Inc. retains all
* ownership rights.
*******************************************************************************
*/

using System;
using NUnit.Framework;

using RPCSupport;
using RPCSupport.Devices;

namespace RPCSupportTest
{
    [TestFixture]
    [Property("Class", "MAX30003")]
    class MAX30003Test
    {
        RPCClient rpcClient;
        MAX30001 max30001;

        [TestFixtureSetUp]
        public void Init()
        {
            rpcClient = new RPCClient(RPCClient.ePipeline.eSerialWrap);
            rpcClient.InitPipeline();
            rpcClient.Connect(ComPort.Port);
            //rpcClient.ShowRpcLog(true);
            max30001 = rpcClient.Init_MAX30001();
        }

        [TestFixtureTearDown]
        public void Dispose()
        {
            rpcClient.Disconnect();
        }

        [Test]
        public void ReadReg()
        {
            int data = max30001.ReadReg(0x0F);

            Assert.AreEqual(0x513174, data);
        }

        [Test]
        public void WriteReg()
        {
            int writeData = 0x123456;

            int data = max30001.ReadReg(0x05);

            max30001.WriteReg(0x05, writeData);

            int result = max30001.ReadReg(0x05);

            max30001.WriteReg(0x05, data);

            Assert.AreEqual(writeData, result);
        }

    }
}
